#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in mat4 modelMatrix;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

//uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
//uniform mat3 normalMatrix;

uniform vec4 animationParams; //parameters for animation: amplitude, frequency, weight for x displacement, weight for y displacement
uniform float elapsedTime;

void main() {
	vec3 finalPosition = position;


	vert.normal_world = mat3(modelMatrix) * normal;
	vert.uv = uv;
	
	vec4 position_world_ = modelMatrix * vec4(position, 1);

	if (animationParams.x != 0) {
		
		float heightX = sin(animationParams.y * elapsedTime + animationParams.z * position_world_.x);
		float heightZ = cos(animationParams.y * elapsedTime + animationParams.w * position_world_.z);
		float height = animationParams.x * heightX * heightZ;

		float xSlope = animationParams.x * cos(animationParams.y * elapsedTime + animationParams.z * position_world_.x) * animationParams.z * heightZ; // d height / d x
		float zSlope = animationParams.x * heightX * -sin(animationParams.y * elapsedTime + animationParams.w * position_world_.z) * animationParams.w; // d height / d z

		vec3 xTangent = vec3(1, xSlope, 0);
		vec3 zTangent = vec3(0, zSlope, 1);

		vec3 normal = cross(zTangent, xTangent);

		vert.normal_world = normalize(normal);

		position_world_.xyz += height * vec3(0,1,0);

	}

	vert.position_world = position_world_.xyz;
	gl_Position = viewProjMatrix * position_world_;
}